import { Body, Controller, Inject, Post } from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
import { BizCode } from '../biz/code';
import {
  adminSign,
  adminSignExpired,
  webSign,
  webSignExpired,
} from '../config/base.config';
import { UserAdminAuthDTO, UserWebAuthDTO } from '../dto/user.dto';
import { UserService } from '../service/user.service';
import { createToken, md5 } from '../util/encrypt';

@Controller('/user')
export class UserController {
  @Inject()
  ctx: Context;

  @Inject()
  userService: UserService;

  /**
   * 用户前台登录
   */
  @Post('/web/auth')
  async webAuth(@Body() params: UserWebAuthDTO) {
    try {
      const userExist = await this.userService.select(params);
      if (userExist?.id) {
        const { user_pass, ...rest } = userExist;
        const passValid = userExist.user_pass === md5(params.user_pass);
        const token = createToken({ ...rest, hasLogin: true });
        if (passValid) {
          this.ctx.cookies.set(webSign, token, {
            expires: new Date(Date.now() + webSignExpired),
            httpOnly: false,
          });
          return {
            code: BizCode.OK,
            msg: '欢迎回来',
            data: { ...rest },
          };
        } else {
          return { code: BizCode.ERROR, msg: '密码错误' };
        }
      } else {
        const createUser = await this.userService.save(params);
        const { user_pass, ...rest } = createUser;
        const token = createToken({ ...rest, hasLogin: true });
        this.ctx.cookies.set(webSign, token, {
          expires: new Date(Date.now() + webSignExpired),
          httpOnly: false,
        });
        return {
          code: BizCode.OK,
          data: { ...rest },
          msg: '欢迎来到 backset.cn',
        };
      }
    } catch (error) {
      this.ctx.logger.error(error);
      return { code: BizCode.ERROR, msg: '[error] web/auth error' };
    }
  }

  /**
   * 管理员登录
   */
  @Post('/admin/auth')
  async AdminAuth(@Body() params: UserAdminAuthDTO) {
    const { username, password } = params;
    const token = createToken({ hasLogin: true });
    if (username === 'admin' && password === '123123') {
      this.ctx.cookies.set(adminSign, token, {
        expires: new Date(Date.now() + adminSignExpired),
        httpOnly: false,
      });
      return { code: BizCode.OK };
    } else {
      return { code: BizCode.ERROR, msg: '用户名密码错误' };
    }
  }
}
